# -*- coding: utf-8 -*-

# Provides a central class for data 
import json
import os
import fractions
import math
#  import numpy as np
import copy
import string
import gzip
#  import pdb


from swat_em import analyse


class datamodel:
    '''
    Provides a central place for all data. All analysis functions are
    connect with this class.
    ''' 
    file_format = 1
    machinedata_keys = ['Q', 'p', 'm', 'phases', 'wstep']
    results_keys = ['q', 'nu_el', 'Ei_el', 'kw_el', 'phaseangle_el', 
                    'nu_mech', 'Ei_mech', 'kw_mech', 'phaseangle_mech',
                    'valid', 'error']
    
    def __init__(self):
        self.reset_data()
        self.reset_results()
        #  self.reset_config()
        self.filename = None
        self.actual_state_saved = False
        
    def set_filename(self, filename):
        self.filename = filename
    
    def reset_data(self):
        '''
        resets all data of the datamodel
        '''
        self.machinedata = {}
        for key in self.machinedata_keys:
            self.machinedata[key] = None
        self.actual_state_saved = False
    
    def reset_results(self):
        self.results = {}
        for key in self.results_keys:
            self.results[key] = None
        self.actual_state_saved = False
    
    def set_config(self, config):
        self.config = config
    
    def set_machinedata(self, Q = None, p = None, m = None):
        '''
        setting the machine data 

        Parameters
        ----------
        Q :      integer
                 number of slots
        p :      integer
                 number of pole pairs
        m:       integer
                 number of phases
        '''
        if Q:
            self.machinedata['Q'] = int(Q)
        if p:
            self.machinedata['p'] = int(p)
        if m:
            self.machinedata['m'] = int(m)
        self.actual_state_saved = False
        
        
    def set_phases(self, S, wstep = None):
        '''
        setting the winding layout 

        Parameters
        ----------
        S : list of lists
            winding layout for every phase, for example:
            S = [[1,-2], [3,-4], [5,-6]]. This means there are 3 phases
            with phase 1 in in slot 1 and in slot 2 with negativ winding
            direction.
        wstep : winding step (slots as unit)
        ''' 
        self.machinedata['phases'] = S
        self.actual_state_saved = False
        self.set_machinedata(m = len(S))
        self.machinedata['phasenames'] = [string.ascii_uppercase[k] for k in range(len(S))]
        if wstep:
            self.machinedata['wstep'] = int(wstep)
        

    def set_valid(self, valid, error):
        self.results['valid'] = valid
        self.results['error'] = error


    def get_num_layers(self):
        '''
        Returns the number of layers of the actual winding layout
        '''
        l = 1
        for p in self.machinedata['phases']:
            if len(p[1]) > 1:
                l = 2
        return l

        
    def get_basic_characteristics(self):
        '''
        Returns the basic charactericits of the winding as 
        dictionary and a string
        '''
        if not 'basic_char' in self.results.keys():
            bc = analyse.get_basic_characteristics(
                self.machinedata['Q'],
                2*self.machinedata['p'],
                self.machinedata['m'],
                self.machinedata['phases'])
            self.results['basic_char'] = bc
        else:
            bc = self.results['basic_char']
        txt = []
        txt.append('q: ' + str(bc['q']))
        txt.append('periodic: ' + str(bc['a']))
        
        txt.append('kw1: ')
        for k in bc['kw1']:
            txt[-1] += ( str(round(k, 3)) + ', ' )
        txt[-1] = txt[-1][:-2]
        
        txt.append('lcm(Q,P): ' + str(bc['lcmQP']))
        
        txt.append('periodic base winding t: ' + str(bc['t']))
        txt.append('parallel conection a: ' + str(bc['a']))
        if bc['sym'] and bc['a']:
            txt.append('symmetric: ' + str(bc['sym']))
        else:            
            #  txt.append('<span style=\" font-size:8pt; font-weight:600; color:#ff0000;\" >')
            t = '<span style=\" color:#ff0000;\" >'
            t += 'symmetric: ' + str(bc['sym'])
            t += '</span>'
            txt.append(t)

        if self.results['error']:
            t = '<span style=\" color:#ff0000;\" >'
            t += 'ERROR: ' + str(self.results['error'])
            t += '</span>'
            txt.append(t)

        
        txt = '<br>'.join(txt)  # line break in html
        return bc, txt
        
    def calc_q(self):
        self.results['q'] = analyse.calc_q(self.machinedata['Q'], 
                                self.machinedata['p'],
                                self.machinedata['m'])
        
        
    def analyse_wdg(self):
        '''
        analyses the winding (winding factor etc.)
        ''' 
        self.reset_results()
        
        # number of slots per pole and phase
        self.q = fractions.Fraction(self.machinedata['Q'] / (
            self.machinedata['m']*2*self.machinedata['p'])).limit_denominator(100)
        self.results['q'] = self.q
        self.n = self.q.denominator
        self.z = self.q.numerator
        
        # base winding
        self.results['t'] = math.gcd(self.machinedata['Q'], self.machinedata['p'])
        
        # electrical winding factor
        a, b, c, d = analyse.calc_kw(
            self.machinedata['Q'], 
            self.machinedata['phases'], 
            self.machinedata['p'], 
            self.config['N_nu_el'],
            self.config )
        self.results['nu_el'] = a
        self.results['Ei_el'] = b
        self.results['kw_el'] = c
        self.results['phaseangle_el'] = d
        
        # mechanical winding factor
        a, b, c, d = analyse.calc_kw(
            self.machinedata['Q'], 
            self.machinedata['phases'], 
            1.0, 
            self.config['N_nu_mech'],
            self.config )
        self.results['nu_mech'] = a
        self.results['Ei_mech'] = b
        self.results['kw_mech'] = c
        self.results['phaseangle_mech'] = d
        
        # winding symmetric?
        self.results['wdg_is_symmetric'] = analyse.wdg_is_symmetric(self.results['Ei_el'],
            self.machinedata['m'])
        #  print(self.results['wdg_is_symmetric'])
        
        # periodicity of the winding (radial force, parallel connection)? 
        self.results['wdg_periodic'] = analyse.wdg_get_periodic(self.results['Ei_el'], self.machinedata['phases'])
        #  print(self.results['wdg_periodic'])
        self.actual_state_saved = False
        
        # MMK
        phi, MMK, theta = analyse.calc_MMK(self.machinedata['Q'],
                                           self.machinedata['m'],
                                           self.machinedata['phases'])
        nu = list(range(self.config['max_nu_MMK']+1))
        HA = analyse.DFT(MMK[:-1])[:self.config['max_nu_MMK']+1]
        self.results['MMK'] = {}
        self.results['MMK']['MMK'] = MMK
        self.results['MMK']['phi'] = phi
        self.results['MMK']['theta'] = theta
        self.results['MMK']['nu'] = nu
        self.results['MMK']['HA'] = HA.tolist()
        
        bc, bc_txt = self.get_basic_characteristics()
        self.results['basic_char'] = bc

        

    def save_to_file(self, fname):
        '''
        Saves the data to file. 

        Parameters
        ----------
        fname :  string
                 file name
        ''' 
        def complex2tuple_1(cplx):
            '''
            converts complex data to tuple of (real, imag)-part
            
            Parameters
            ----------
            fname : list
                    with complex numbers
            '''
            for k1 in range(len(cplx)):
                        cplx[k1] = (cplx[k1].real, cplx[k1].imag)
            return cplx
       
        def complex2tuple_3(cplx):
            '''
            converts complex data to tuple of (real, imag)-part
            
            Parameters
            ----------
            fname : list(list(list))
                    3 nested lists with complex numbers
            '''
            for k1 in range(len(cplx)):
                for k2 in range(len(cplx[k1])):
                    for k3 in range(len(cplx[k1][k2])):
                        cplx[k1][k2][k3] = (cplx[k1][k2][k3].real, cplx[k1][k2][k3].imag)
            return cplx
        
        M = {}
        M['file_format'] = self.file_format
        M['machinedata'] = self.machinedata
        #  M['config'] = self.config
        M['results'] = copy.deepcopy(self.results)
        
        # convert lists with complex numbers to tuple with real and imag
        if 'Ei_el' in M['results'].keys():
            M['results']['Ei_el'] = complex2tuple_3(M['results']['Ei_el'])
        
        if 'Ei_mech' in M['results'].keys():
            M['results']['Ei_mech'] = complex2tuple_3(M['results']['Ei_mech'])
        
        if 'MMK' in M['results'].keys():
            M['results']['MMK']['HA'] = complex2tuple_1(M['results']['MMK']['HA'])
        
        # convert fractional number
        if 'q' in M['results'].keys():
            M['results']['q'] = str(M['results']['q'])
            
        if 'basic_char' in M['results'].keys():
            if 'q' in M['results']['basic_char'].keys():
                M['results']['basic_char']['q'] = str(M['results']['basic_char']['q'])
        
        # save as compressed file
        with gzip.GzipFile(fname, 'w') as f:
            s = json.dumps(M, indent = 2)                  # string
            f.write(s.encode('utf-8'))           # bytes        
            
        # save as ASCII file
        #  with open(fname, 'w') as f:
            #  json.dump(M, f, indent = 2)
        self.actual_state_saved = True


    def load_from_file(self, fname):
        '''
        Load data from file. 

        Parameters
        ----------
        fname :  string
                 file name
        ''' 
        
        def tuple2complex_1(cplx):
            '''
            converts tuple of (real, imag) to complex number
            
            Parameters
            ----------
            fname : list
                    with complex numbers
            '''
            for k1 in range(len(cplx)):
                        cplx[k1] = cplx[k1][0] + 1j*cplx[k1][1]
            return cplx
        
        def tuple2complex_3(cplx):
            '''
            converts tuple of (real, imag) to complex number
            
            Parameters
            ----------
            fname : list(list(list))
                    3 nested lists with tuples
            '''
            for k1 in range(len(cplx)):
                for k2 in range(len(cplx[k1])):
                    for k3 in range(len(cplx[k1][k2])):
                        cplx[k1][k2][k3] = cplx[k1][k2][k3][0] + 1j*cplx[k1][k2][k3][1]
            return cplx
        
        
        if os.path.isfile(fname):
            self.reset_results()
            # load ASCII file
            #  with open(fname) as f:
                #  M = json.load(f)
            
            # load compressed file
            with gzip.GzipFile(fname) as f:  # gzip
                s = f.read()                    # bytes
                s = s.decode('utf-8')           # string
                M = json.loads(s)               # data  
            
            self.machinedata = M['machinedata']
            
            # load config
            #  self.config = M['config']
            
            # convert lists with tuple of real and imag to complex number
            if 'Ei_el' in M['results'].keys():
                M['results']['Ei_el'] = tuple2complex_3(M['results']['Ei_el'])
            
            if 'Ei_mech' in M['results'].keys():
                M['results']['Ei_mech'] = tuple2complex_3(M['results']['Ei_mech'])
            
            if 'MMK' in M['results'].keys():
                M['results']['MMK']['HA'] = tuple2complex_1(M['results']['MMK']['HA'])
            
            # convert fractional number
            if 'q' in M['results'].keys():
                M['results']['q'] = fractions.Fraction(M['results']['q'])
            
            if 'basic_char' in M['results'].keys():
                if 'q' in M['results']['basic_char'].keys():
                    M['results']['basic_char']['q'] = fractions.Fraction(M['results']['basic_char']['q'])
            
                self.results = copy.deepcopy(M['results'])
            self.actual_state_saved = True













# -*- coding: utf-8 -*-



from PyQt5 import uic
from PyQt5.QtWidgets import QMainWindow, QApplication, QFileDialog, QMessageBox
from PyQt5.QtGui import QIntValidator
#  from PyQt5.QtCore import QFile, QTextStream
import sys
import os
import time

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
from swat_em import dialog_genwdg
from swat_em import dialog_about
from swat_em import dialog_winding_layout
from swat_em import dialog_factors
from swat_em import dialog_settings
from swat_em.config import get_config, save_config

#  import pdb


from swat_em import datamodel
from swat_em import wdggenerator
from swat_em import plots



__dir__ = os.path.dirname(os.path.abspath(__file__))


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        
        self.data = datamodel.datamodel()
        self.config = get_config(default = False)
        self.data.set_config(self.config)

        # Set up the user interface from Designer.
        uic.loadUi(os.path.join(__dir__, 'ui/MainWindow.ui'), self)
        self.setWindowTitle('SWAT')

        self.DIALOG_GenWinding = dialog_genwdg.GenWinding2(self.config)
        self.DIALOG_GenWindingCombinations = dialog_genwdg.GenWindingCombinations(self.config)
        self.DIALOG_AdditionalFactors = dialog_factors.Factors(self.config, self.data)
        

        # init line edits
        #  self.lineEdit_Q.setValidator(QIntValidator()) # only integers allowed
        #  self.lineEdit_P.setValidator(QIntValidator())
        #  self.lineEdit_m.setValidator(QIntValidator())
        #  self.lineEdit_Q.textChanged.connect(self.lineEdit_Q_changed)

        # Connect up the buttons.
        self.Button_exit.clicked.connect(self.close)
        self.Button_EditWindingLayout.clicked.connect(self.dialog_EditWindingLayout)
        self.Button_GenerateAutomatically.clicked.connect(self.dialog_GenWinding)
        self.Button_FindByTable.clicked.connect(self.dialog_GenWindingCombinations)
        
        
        # Connect menu
        self.actionExit.triggered.connect(self.close)
        self.actionGenerate_winding.triggered.connect(self.dialog_GenWinding)
        self.actionGenerate_winding_combinations.triggered.connect(self.dialog_GenWindingCombinations)
        
        self.actionsave.triggered.connect(self.save_to_file)
        self.actionsave_as.triggered.connect(self.save_as_to_file)
        self.actionload.triggered.connect(self.load_from_file)
        self.actionProgram_Info.triggered.connect(dialog_about.about)
        self.actionShow_winding_layout.triggered.connect(self.dialog_EditWindingLayout)
        self.actionAdditional_factors.triggered.connect(self.dialog_AdditionalFactors)
        self.actionSettings.triggered.connect(self.dialog_settings)
        
        # TODO
        self.actionprint_report.triggered.connect(lambda: QMessageBox.information(self, 'Information', 'Not implemented yet'))
        self.actionHelp.triggered.connect(lambda: QMessageBox.information(self, 'Information', 'Not implemented yet'))
        
        self.plot_tabs.currentChanged.connect(self.update_plot_in_GUI)
        self.radioButton_electrical.toggled.connect(self.update_plot_in_GUI)
        self.comboBox_star_harmonics.currentIndexChanged.connect(self.update_plot_in_GUI)
        self.checkBoxForceX.toggled.connect(self.update_plot_in_GUI)
        


        #  initial winding  -----------------------------------
        self.data.set_machinedata(Q = 12, p = 10/2, m = 3)
        wdglayout = wdggenerator.genwdg(
                            self.data.machinedata['Q'], 
                            2*self.data.machinedata['p'], 
                            self.data.machinedata['m'], 
                            w = 1, 
                            layers = 2)
        #  ----------------------------------------------------
        
        self.data.set_phases(S = wdglayout['phases'], wstep = wdglayout['wstep'])
        self.data.set_valid(valid = wdglayout['valid'], error = wdglayout['error'])
        self.data.analyse_wdg()
        self.data.actual_state_saved = True # Simulate save state
        
        
        # Plots
        self.fig1 = plots.slot_plot(self.mplvl_slot, self.mplwidget_slot, self.data)
        self.fig2 = plots.slot_star(self.mplvl_star, self.mplwidget_star, self.data, self.tableWidget_star)
        self.fig3 = plots.windingfactor(self.mplvl_wf, self.mplwidget_wf, self.data, self.tableWidget_wf)
        self.fig4 = plots.mmk(self.mplvl_mmk, self.mplwidget_mmk, self.data, self.tableWidget_mmk)
        
        self.update_data_in_GUI()
        
        self.show()
        
    


    def dialog_GenWinding(self):
        '''
        calls the dialog to generate a single winding
        '''
        ret = self.DIALOG_GenWinding.run()
        if ret:
            wdglayout = wdggenerator.genwdg(ret['Q'], ret['P'], ret['m'], ret['w'], ret['layers'])
            self.data.set_machinedata(Q = ret['Q'], p = ret['P']//2, m = ret['m'])
            self.data.set_phases(wdglayout['phases'], wstep = wdglayout['wstep'])            
            self.data.analyse_wdg()
            self.data.set_valid(valid = wdglayout['valid'], error = wdglayout['error'])            
            self.update_data_in_GUI()

    def dialog_GenWindingCombinations(self):
        '''
        calls the dialog to generate a winding based on a big table of available windings
        '''
        ret = self.DIALOG_GenWindingCombinations.run()
        if ret:
            wdglayout = wdggenerator.genwdg(ret['Q'], ret['P'], ret['m'], ret['w'], ret['layers'])
            self.data.set_machinedata(Q = ret['Q'], p = ret['P']//2, m = ret['m'])
            self.data.set_phases(wdglayout['phases'], wstep = wdglayout['wstep'])            
            self.data.analyse_wdg()
            self.data.set_valid(valid = wdglayout['valid'], error = wdglayout['error'])            
            self.update_data_in_GUI()


    def dialog_EditWindingLayout(self):
        '''
        calls the dialog to generate a winding based on a big table of available windings
        '''
        DIALOG_EditWindingLayout = dialog_winding_layout.layout(self.data)
        ret = DIALOG_EditWindingLayout.run()

        if ret:
            self.data.set_machinedata(Q = ret['Q'], p = ret['P']//2, m = ret['m'])
            self.data.set_phases(ret['phases'], wstep = ret['w'])            
            self.data.analyse_wdg()
            #  self.data.set_valid(valid = wdglayout['valid'], error = wdglayout['error'])            
            self.update_data_in_GUI()


    def dialog_AdditionalFactors(self):
        ret = self.DIALOG_AdditionalFactors.run()


    def dialog_settings(self):
        DIALOG_Settings = dialog_settings.Settings(self.config)
        ret = DIALOG_Settings.run()
        if ret:
            self.config = ret
            save_config(self.config)
            self.data.analyse_wdg()   # recalc the winding model
            self.update_data_in_GUI()



    def update_data_in_GUI(self):
        '''
        plots all winding data in gui
        '''
        t1 = time.time()
        # Update line edits
        #  self.lineEdit_P.blockSignals(True)  # turn off signals to prevent trigger during update
        #  self.lineEdit_P.setText(str(int(self.data.machinedata['p']*2)))
        #  self.lineEdit_P.blockSignals(False)
        
        #  self.lineEdit_m.blockSignals(True)
        #  self.lineEdit_m.setText(str(self.data.machinedata['m']))
        #  self.lineEdit_m.blockSignals(False)
        
        #  self.lineEdit_Q.blockSignals(True)
        #  self.lineEdit_Q.setText(str(self.data.machinedata['Q']))
        #  self.lineEdit_Q.blockSignals(False)
        
        
        
        # Update machine infos
        # TODO: fundamental windingfactor        
        bc, bc_str = self.data.get_basic_characteristics()
        #  self.textBrowser_wdginfo.setPlainText(bc_str)
        self.textBrowser_wdginfo.setHtml(bc_str)
        self.update_plot_in_GUI()
        
        self.comboBox_star_harmonics.blockSignals(True)   # prevent double plotting on startup
        self.comboBox_star_harmonics.clear()
        self.comboBox_star_harmonics.addItems([str(k) for k in self.data.results['nu_el']])
        self.comboBox_star_harmonics.blockSignals(False)
        
        
    def update_plot_in_GUI(self):
        
        # Update Figures
        idx = self.plot_tabs.currentIndex()
        # update only the current tab
        if idx == 0:
            self.fig1.plot_slots(self.data.machinedata['Q'])
            #  print('time for plotting:', time.time()-t1); t1 = time.time()
            self.fig1.plot_coilsides()
            #  print('time for plotting:', time.time()-t1); t1 = time.time()
        if idx == 1: 
            
            self.fig2.plot_star(harmonic_idx = self.comboBox_star_harmonics.currentIndex(), ForceX = self.checkBoxForceX.isChecked())
            #  print('time for plotting:', time.time()-t1); t1 = time.time()
        if idx == 2:
            if self.radioButton_electrical.isChecked():
                self.fig3.plot_windingfactor(mechanical=False)
            elif self.radioButton_mechanical.isChecked():
                self.fig3.plot_windingfactor(mechanical=True)
            #  print('time for plotting:', time.time()-t1); t1 = time.time()
        if idx == 3:  
            self.fig4.plot_mmk()
            #  print('time for plotting:', time.time()-t1); t1 = time.time()

        
        
    def save_to_file(self):
        '''
        returns True if file is saved
        '''
        if not self.data.filename:
            ret = self.save_as_to_file()
            return ret
        else:
            self.data.save_to_file(self.data.filename)
            return True


    def save_as_to_file(self):
        '''
        returns True if file is saved
        '''
        options = QFileDialog.Options()
        #  options |= QFileDialog.DontUseNativeDialog
        filename, _ = QFileDialog.getSaveFileName(self,"Save winding file", "","Winding Files (*.wdg)", options=options)
        if filename:
            if not filename.endswith('.wdg'):
                filename += '.wdg'
            self.data.set_filename(filename) 
            self.data.save_to_file(self.data.filename)
            return True
        else:
            return False


    def load_from_file(self):
        if not self.data.actual_state_saved:
            #  print('ask for saving')
            ok = QMessageBox.question(self, 'Exit program', "Do you want to save?", QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel, QMessageBox.Cancel)
            if ok == QMessageBox.Yes:
                ret = self.save_to_file()
                if not ret: # user has not saved
                    return
            if ok == QMessageBox.Cancel: # cancel
                return
            
        
        options = QFileDialog.Options()
        #  options |= QFileDialog.DontUseNativeDialog  # use qt5 dialog instead of os default
        filename, _ = QFileDialog.getOpenFileName(self,"Open winding file", "","Winding Files (*.wdg)", options=options)
        if filename:
            self.data.load_from_file(filename)
            self.data.set_filename(filename)
            self.update_data_in_GUI()
        
    
    def closeEvent(self, event):
        # Exit programm
        
        # Ask for saving
        if self.data.actual_state_saved:
            event.accept() # let the window close
        else:
            #  print('ask for saving')
            ok = QMessageBox.question(self, 'Exit program', "Do you want to save?", QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel, QMessageBox.Cancel)
            if ok == QMessageBox.Yes:
                ret = self.save_to_file()
                if ret:  # user has saved
                    event.accept() # let the window close
                else:    # user has not saved
                    event.ignore()
            elif ok == QMessageBox.No:
                event.accept() # close the window
            else:
                event.ignore() # let the window open
                


def main():
    
    # TODO: Eval command line arguments
    
    app = QApplication(sys.argv)
    #  file = QFile('themes/qdarkstyle_martin.qss')
    #  file = QFile('themes/dark.qss')
    #  file = QFile('themes/test.qss')
    #  file = QFile('themes/DarkMonokai.qss')
    
    #  file.open(QFile.ReadOnly | QFile.Text)
    #  stream = QTextStream(file)
    #  app.setStyleSheet(stream.readAll())
    ex = MainWindow()
    sys.exit(app.exec_())



if __name__ == '__main__':
    main()
    
    
    



# -*- coding: utf-8 -*-

import os
import sys
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

import swat_em.config

# TODO: Test the config save without overwriting the users config in home folder!
#  def test1():
    #  config = swat_em.config.get_config()
    #  swat_em.config.save_config(config)
    #  config2 = swat_em.config.get_config(init = False)
    #  assert config == config2

#  if __name__ == '__main__':
    #  test1()









# -*- coding: utf-8 -*-
"""


"""
import os
__title__ = 'SWAT-EM - Specific Winding Analyse Tool for electrical machines'
__version__ = '0.1.1'
__author__ = 'Martin Baun'
__license__ = 'GPLv3'
__copyright__ = 'Copyright 2018-2019 Martin Baun'

#  from .gui import MainWindow
#  import swat.gui as gui
__dir__ = os.path.dirname(os.path.abspath(__file__))

#  from PyQt5.QtWidgets import  QApplication
#  import sys


#  def run():
    #  app = QApplication(sys.argv)
    #  ex = gui.MainWindow()
    #  sys.exit(app.exec_())

    



# -*- coding: utf-8 -*-

import os
import sys
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

#  from swat.wdggenerator import genwdg, overlapping_fractional_slot_slayer, overlapping_fractional_slot_dlayer
from swat_em.wdggenerator import genwdg
from swat_em.datamodel import datamodel
from swat_em.config import get_init_config
import swat_em.analyse
import numpy as np
import pdb



def test1():
    print('Test if MMK results in the same windingfactor as star of voltage')
    print('double layer winding, tooth-coil')
    Q = 18
    p = 8
    m = 3
    wstep = 1
    layers = 2
    ret = genwdg(Q, 2*p, m, wstep, layers)
    wdglayout = ret['phases']
    data = datamodel()
    data.set_config(get_init_config())
    data.set_machinedata(Q = Q, p = p, m = m)
    data.set_phases(wdglayout)
    data.analyse_wdg()

    nu = data.results['MMK']['nu']
    HA = data.results['MMK']['HA']
    idx = nu.index(p)
    
    Ns = Q*layers/m/2   # number of turns in series
    kw1 = np.abs(HA[idx])*np.pi*p / (3.*Ns)  # winding-factor based on the MMK
        
    idx = data.results['nu_el'].index(1)
    assert np.round(kw1, 4) == np.round(data.results['kw_el'][idx][0], 4) # phase U
    
    #  import matplotlib.pyplot as plt
    #  try: # Interaktives Plotten, wenn Skript aus der ipython Konsole aufgerufen wird
        #  __IPYTHON__
        #  plt.ion()
    #  except NameError:
        #  pass
    #  plt.figure(1)
    #  plt.clf()
    #  plt.grid(True)
    #  plt.stem(range(len(theta)), theta, 'r', markerfmt=' ') #, basefmt=' '
    #  plt.plot(phi, MMK)



def test2():
    print('Test if MMK results in the same windingfactor as star of voltage')
    print('single layer winding, tooth-coil')
    Q = 12
    p = 5
    m = 3
    wstep = 1
    layers = 1
    ret = genwdg(Q, 2*p, m, wstep, layers)
    wdglayout = ret['phases']
    data = datamodel()
    data.set_config(get_init_config())
    data.set_machinedata(Q = Q, p = p, m = m)
    data.set_phases(wdglayout)
    data.analyse_wdg()

    nu = data.results['MMK']['nu']
    HA = data.results['MMK']['HA']
    idx = nu.index(p)
    
    Ns = Q*layers/m/2   # number of turns in series
    kw1 = np.abs(HA[idx])*np.pi*p / (3.*Ns)  # winding-factor based on the MMK
        
    idx = data.results['nu_el'].index(1)
    assert np.round(kw1, 4) == np.round(data.results['kw_el'][idx][0], 4) # phase U
    
    #  import matplotlib.pyplot as plt
    #  try: # Interaktives Plotten, wenn Skript aus der ipython Konsole aufgerufen wird
        #  __IPYTHON__
        #  plt.ion()
    #  except NameError:
        #  pass
    #  plt.figure(1)
    #  plt.clf()
    #  plt.grid(True)
    #  plt.stem(range(len(theta)), theta, 'r', markerfmt=' ') #, basefmt=' '
    #  plt.plot(phi, MMK)


def test3():
    print('Test if MMK results in the same windingfactor as star of voltage')
    print('double layer winding, overlapping')
    Q = 18
    p = 1
    m = 3
    wstep = 6
    layers = 2
    ret = genwdg(Q, 2*p, m, wstep, layers)
    wdglayout = ret['phases']
    data = datamodel()
    data.set_config(get_init_config())
    data.set_machinedata(Q = Q, p = p, m = m)
    data.set_phases(wdglayout)
    data.analyse_wdg()

    nu = data.results['MMK']['nu']
    HA = data.results['MMK']['HA']
    idx = nu.index(p)
    
    Ns = Q*layers/m/2   # number of turns in series
    kw1 = np.abs(HA[idx])*np.pi*p / (3.*Ns)  # winding-factor based on the MMK
        
    idx = data.results['nu_el'].index(1)
    assert np.round(kw1, 4) == np.round(data.results['kw_el'][idx][0], 4) # phase U
    
    #  import matplotlib.pyplot as plt
    #  try: # Interaktives Plotten, wenn Skript aus der ipython Konsole aufgerufen wird
        #  __IPYTHON__
        #  plt.ion()
    #  except NameError:
        #  pass
    #  plt.figure(1)
    #  plt.clf()
    #  plt.grid(True)
    #  plt.stem(range(len(theta)), theta, 'r', markerfmt=' ') #, basefmt=' '
    #  plt.plot(phi, MMK)




if __name__ == '__main__':
    test1()
    test2()
    test3()








# -*- coding: utf-8 -*-
'''
Provides functions for plotting
''' 
#  import time
#  import pdb

from PyQt5.QtWidgets import QTableWidgetItem
from PyQt5.QtGui import QFont
from swat_em.config import get_phase_color
import matplotlib as mpl
import matplotlib.pyplot as plt
#  from matplotlib.figure import Figure
from matplotlib.backends.backend_qt5agg import (
    FigureCanvasQTAgg as FigureCanvas,
    NavigationToolbar2QT as NavigationToolbar)
#  import matplotlib.patches as patches
import numpy as np

#  import time



class slot_plot:
    sh = 0.8  # slot height
    sw = 0.75  # slot width
    so = 0.2  # slot opening

    def __init__(self, layout, widget, data):
        self.layout = layout
        self.widget = widget
        self.data = data
        self.Q = 0
        self.slot = {}
        self.devide = 'h'  # slot devide h (horizontal) or v (vertical
        self.fig = plt.figure(1, dpi=self.data.config['plt']['DPI'])
        self.fig.clf()
        self.canvas1 = FigureCanvas(self.fig)
        self.layout.addWidget(self.canvas1)
        self.canvas1.draw()
        self.toolbar1 = NavigationToolbar(self.canvas1, 
                self.widget, coordinates=True)
        self.toolbar1.setFixedHeight(25)
        self.layout.addWidget(self.toolbar1)
        #  self.cid = self.fig.canvas.mpl_connect('button_press_event', self.onclick_wdglayout)

       
    
    def plot_slots(self, Q):
        if Q <= 0:
            return False
        sh = self.sh  # slot height
        sw = self.sw  # slot width
        so = self.so  # slot opening


        
        plt.figure(1)
        #  self.fig1.clf()
        plt.clf()
        plt.axis('equal')
        self.ax = plt.gca()
        self.ax.axis('off')
        plt.subplots_adjust(left=0, bottom=0, right=1, top=1)
    
        x = [0]*10
        y = [0]*10
        x[0], y[0] = 0.0, 1.0
        x[1], y[1] = 0.5 - so/2, 1.0
        x[2], y[2] = 0.5 - so/2, sh
        x[3], y[3] = 0.5 - sw/2, sh
        x[4], y[4] = 0.5 - sw/2, 0.0
        x[5], y[5] = 0.5 + sw/2, 0.0
        x[6], y[6] = 0.5 + sw/2, sh
        x[7], y[7] = 0.5 + so/2, sh
        x[8], y[8] = 0.5 + so/2, 1.0
        x[9], y[9] = 1.0, 1.0

        xp = [[]]
        yp = [[]]
        i_slots = 0
        dx = 0
        dy = 0
        for k in range(Q):
            if i_slots >= 12:
                dx = 0
                dy += -1.5
                i_slots = 0
                xp.append([])
                yp.append([])
            for a, b in zip(x,y):
                xp[-1].append(a+dx)
                yp[-1].append(b+dy) 
            dx += 1
            i_slots += 1
        for k in range(len(xp)):
            plt.plot(xp[k], yp[k], 'k')
        
        i_slots = 0
        dy = 0
        dx = 0
        for k in range(Q):
            if i_slots >= 12:
                dx = 0
                dy += -1.5
                i_slots = 0
            plt.text(0.5+dx, -0.3+dy, str(k+1), horizontalalignment='center')
            dx += 1
            i_slots += 1
        
        self.canvas1.draw()
    

                        
    def plot_coilsides(self):
        S = self.data.machinedata['phases']
        Q = self.data.machinedata['Q']
        self.devide = 'v' if self.data.machinedata['wstep'] == 1 else 'h'
        
        def add_text(slot, phase, pos, wdir):
            dx = slot
            dy = self.sh / 2
            while dx > 12:
                dx -= 12
                dy -= 1.5
            dx = dx -1 + 0.5

            if self.devide == 'h':
                if pos == 0:         # bottom layer
                    dy -= self.sh/4
                elif pos == 1:       # top layer
                    dy += self.sh/4
                    
            elif self.devide == 'v':
                if pos == 0:         # right layer
                    dx += self.sw/4
                elif pos == 1:       # left layer
                    dx -= self.sw/4
            
            
            if wdir > 0:
                txt = '+' + str(phase)  # '$\otimes$'
            else:
                txt = '-' + str(phase)  # '$\odot$'
            
            t = plt.text(dx, 
                         dy, 
                         txt, horizontalalignment='center',
                         verticalalignment='center',
                         color = get_phase_color(self.data.config, phase-1),
                         rasterized=False,
                         weight = 'bold')
        
        #  t1 = time.time()
        for m, phase in enumerate(S):
            pos = 0
            for layer in phase:
                for cs in layer:
                    add_text(abs(cs), m+1, pos, cs)
                pos += 1
        self.canvas1.draw()
        #  print('duration for txt plot:', time.time()-t1)

   
    #  def onclick_wdglayout(self, event):
        #  Q = self.data.machinedata['Q']
        #  slot_select = int(event.xdata)+1
        #  slot_select = 1 if slot_select < 1 else slot_select
        #  slot_select = Q if slot_select > Q else slot_select
        #  print('selected slot: '+str(slot_select))
        #  print(event.xdata, event.ydata)
        



class slot_star:

    def __init__(self, layout, widget, data, table):
        self.layout = layout
        self.widget = widget
        self.data = data
        self.table = table

        self.fig = plt.figure(2, dpi=self.data.config['plt']['DPI'])
        self.fig.clf()
        plt.subplots_adjust(left=0, bottom=0, right=1, top=1)
        self.canvas1 = FigureCanvas(self.fig)
        self.layout.addWidget(self.canvas1)
        self.canvas1.draw()
        self.toolbar1 = NavigationToolbar(self.canvas1, 
                self.widget, coordinates=True)
        self.toolbar1.setFixedHeight(25)
        self.layout.addWidget(self.toolbar1)
        
    
    def plot_star(self, harmonic_idx = 0, ForceX = None):
        if harmonic_idx < 0:
            harmonic_idx = 0
        
        fig = plt.figure(2)
        plt.clf()
        
        xlim = [0, 0] # find limit manually, because autoscale doesn't work with plt.arrow()
        ylim = [0, 0]
        def plot_vek(vek, color='k', dangle = None):
            '''
            vek: list of complex values
            dangle: All phasors get shifted by dangle
            '''
            if dangle is not None:
                A = np.abs(vek)
                ph = np.angle(vek)
                ph -= dangle
                vek = A*np.exp(1j*ph)
                
            old = 0+0j
            v = np.cumsum(vek)

            for k in range(len(vek)): # find limit manually, because autoscale doesn't work with plt.arrow()
                if min(v.real) < xlim[0]:
                    xlim[0] = min(v.real)
                if max(v.real) > xlim[1]:
                    xlim[1] = max(v.real)
                if min(v.imag) < ylim[0]:
                    ylim[0] = min(v.imag)
                if max(v.imag) > ylim[1]:
                    ylim[1] = max(v.imag)
                
                plt.arrow( old.real, old.imag, vek[k].real, vek[k].imag, head_width=0.08, fc=color, ec=color, linewidth=self.data.config['plt']['lw'])
                old = v[k]
            plt.plot( [0.0, v[-1].real], [0.0, v[-1].imag], '--', color = color, linewidth=self.data.config['plt']['lw'])
        
        if ForceX:
            angle_phase1 = np.angle(np.sum(self.data.results['Ei_el'][harmonic_idx][0]))
        else:
            angle_phase1 = None
        for k in range(len(self.data.results['Ei_el'][harmonic_idx])):
            plot_vek(self.data.results['Ei_el'][harmonic_idx][k], color = get_phase_color(self.data.config, k), dangle = angle_phase1)

        plt.axis('equal')
        ax = plt.gca()
        # ax.autoscale_view()
        ax.axis('off')
        
        # set limit manually, because autoscale doesn't work with plt.arrow()
        plt.xlim(xlim[0]*1.1, xlim[1]*1.1)
        plt.ylim(ylim[0]*1.1, ylim[1]*1.1)
        ax.set_adjustable("box")   # necessary for xlim/ylim with axis('equal')
        
          
        l = ['Phase '+str(k+1) for k in range(self.data.machinedata['m'])]
        leg = plt.legend(l,loc='upper right',labelspacing=0)    # labelspacing: Zeilenabstand
        leg.draw_frame(0)    # Rahmen nicht zeichnen
        self.canvas1.draw()
        
        
        # update table:        
        ei = self.data.results['Ei_el'][harmonic_idx]        
        
        self.table.setRowCount(len(ei))
        self.table.setColumnCount(3)
        
        for km in range(len(ei)):
            A = np.abs( np.sum(ei[km]) )
            ph = np.angle( np.sum(ei[km]) )/np.pi*180
            self.table.setItem(km, 0, QTableWidgetItem(str(km+1)))
            self.table.setItem(km, 1, QTableWidgetItem(str(round(A,1))))
            self.table.setItem(km, 2, QTableWidgetItem(str(round(ph,1))))

        for k1 in range(len(ei)):
            self.table.resizeColumnToContents(k1)


        self.table.setHorizontalHeaderLabels(['m', 'A', 'phase'])
        #  self.table.setVerticalHeaderLabels(['']*np.shape(kw)[0])
        afont = QFont()
        afont.setBold(True)
        
        for k in range(3):
            self.table.horizontalHeaderItem(k).setFont(afont)


class windingfactor:
    def __init__(self, layout, widget, data, table):
        self.layout = layout
        self.widget = widget
        self.data = data
        self.table = table

        self.fig = plt.figure(3, dpi=self.data.config['plt']['DPI'])
        self.fig.clf()
        self.canvas1 = FigureCanvas(self.fig)
        self.layout.addWidget(self.canvas1)
        self.canvas1.draw()
        self.toolbar1 = NavigationToolbar(self.canvas1, 
                self.widget, coordinates=True)
        self.toolbar1.setFixedHeight(25)
        self.layout.addWidget(self.toolbar1)
        
    
    def plot_windingfactor(self, mechanical = False):
        plt.figure(3)
        plt.clf()
        
        if mechanical:
            nu = np.array(self.data.results['nu_mech'])
            kw = np.array(self.data.results['kw_mech'])
        else:
            nu = np.array(self.data.results['nu_el'])
            kw = np.array(self.data.results['kw_el'])
        
        
        #  gap = 0.2
        N = np.shape(kw)[1]
        for k in range(N):
            dx = nu[1] - nu[0]
            x = nu - dx/N/1.5 + dx/N/1.5*k
            plt.bar( x, np.abs(kw[:,k]), (dx)/1.5/N, label = 'Phase '+str(k+1),
                color = get_phase_color(self.data.config, k))

        #  plt.title('Windingfactor')
        plt.xlabel('ordinal number $\\nu$')
        plt.ylabel('Windingfactor $|\\xi_w|$')
        plt.grid(True)
        ax = plt.gca()
        ax.set_axisbelow(True)  # grid in background        
        leg = plt.legend(loc='upper right',labelspacing=0)    # labelspacing: Zeilenabstand
        #  ltext = leg.get_texts() # all the text.Text instance in the legend
        #  plt.setp(ltext, fontsize='small') # the legend text fontsize
        #  plt.tight_layout()
        self.canvas1.draw()
        
        # update table:
        self.table.setRowCount(np.shape(kw)[0])
        self.table.setColumnCount(N+1)
        
        for k1 in range(np.shape(kw)[0]):
            self.table.setItem(k1, 0, QTableWidgetItem(str(nu[k1])))
            for k2 in range(np.shape(kw)[1]):
                self.table.setItem(k1, k2+1, QTableWidgetItem(str(round(kw[k1,k2],3))))
        for k1 in range(np.shape(kw)[0]):
            self.table.resizeColumnToContents(k1)


        self.table.setHorizontalHeaderLabels(['nu'] + self.data.machinedata['phasenames'])
        self.table.setVerticalHeaderLabels(['']*np.shape(kw)[0])
        afont = QFont()
        afont.setBold(True)
        
        for k in range(self.data.machinedata['m'] + 1):
            self.table.horizontalHeaderItem(k).setFont(afont)


class mmk:

    def __init__(self, layout, widget, data, table):
        self.layout = layout
        self.widget = widget
        self.data = data
        self.table = table

        self.fig = plt.figure(4, dpi=self.data.config['plt']['DPI'])
        self.fig.clf()
        plt.subplots_adjust(left=0, bottom=0, right=1, top=1)
        self.canvas1 = FigureCanvas(self.fig)
        self.layout.addWidget(self.canvas1)
        self.canvas1.draw()
        self.toolbar1 = NavigationToolbar(self.canvas1, 
                self.widget, coordinates=True)
        self.toolbar1.setFixedHeight(25)
        self.layout.addWidget(self.toolbar1)
        
    
    def plot_mmk(self):
        '''
        Plottet MMF-Kurve

        Parameters
        ----------
        plot_MMK_nu :           iterable - list/tuple
                                Liste/Tuple mit Ordnungszahlen, die geplottet werden sollen
        plot_MMK_greater_than : scalar, float
                                Plotte alle Oberschwingungen mit der Amplitude größer als
        '''
        
        plot_MMK_greater_than = 0.15

        plt.figure(4)
        plt.clf()
        
        MMK =  self.data.results['MMK']['MMK']
        phi = np.array(self.data.results['MMK']['phi'])
        theta = self.data.results['MMK']['theta']           

        plt.plot( phi, MMK, linewidth = self.data.config['plt']['lw'], label = 'MMF')
        
        
        #  plt.xlim(xmin=0, xmax=max(phi/np.pi*180))
        plt.grid(True)
        #  plt.title('Felderregerkurve')
        plt.ylabel('MMF in A')
        plt.xlabel('circumferential Stator in deg')

        # Plotte Oberschwingungen
        nu = np.array(self.data.results['MMK']['nu'])
        HA = self.data.results['MMK']['HA']

        A = np.abs(HA)
        phase = np.angle(HA)

        for n, a, p in zip(nu, A, phase):
            if 1/max(A)*a > plot_MMK_greater_than:
                plt.plot( phi, a*np.cos(n*phi/phi[-1]*2*np.pi + p), '--', label='$\\nu={}$'.format(n), linewidth=self.data.config['plt']['lw_thin'])

        leg = plt.legend(loc='upper right',labelspacing=0)
        plt.xlim(min(phi), max(phi))
        
        #  ax = plt.gca()
        #  ax2 = ax.twinx()
        #  ax2.stem(range(len(theta)), theta, 'r', markerfmt=' ')            

        plt.tight_layout()
        self.canvas1.draw()
        
        
        # update table:
        self.table.setRowCount(len(A)-1)
        self.table.setColumnCount(3)
        
        self.table.setHorizontalHeaderLabels(['nu', 'Amp', '[%]'])
        #  self.table.setVerticalHeaderLabels(['']*np.shape(kw)[0])
        
        
        for k1 in range(1, len(A)):
            self.table.setItem(k1-1, 0, QTableWidgetItem(str(nu[k1])))
            a = A[k1]
            a_rel = 100.0/max(A)*a
            self.table.setItem(k1-1, 1, QTableWidgetItem(str(round(a, 3))))
            self.table.setItem(k1-1, 2, QTableWidgetItem(str(round(a_rel, 1))))
            #  for k2 in range(np.shape(kw)[1]):
                #  self.table.setItem(k1, k2+1, QTableWidgetItem(str(round(kw[k1,k2],3))))
        for k1 in range(3):
            self.table.resizeColumnToContents(k1)


        
        #  afont = QFont()
        #  afont.setBold(True)
        
        #  for k in range(self.data.machinedata['m'] + 1):
            #  self.table.horizontalHeaderItem(k).setFont(afont)



import setuptools
import swat_em

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="swat-em",
    version=swat_em.__version__,
    author=swat_em.__author__,
    author_email="mar.baun@googlemail.com",
    description="Specific Winding Analysing Tool for Electrical Machines",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/martinbaun/swat-em",
    packages=setuptools.find_packages(),
    #  packages=['swat_em'],
    package_data={'swat_em': ['themes/*', 'ui/*', 'ui/icons/*', 'ui/icons/Qt/*']},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={"console_scripts": [
    "swat-em = swat_em.main:main"]},
)
